<html py:extends="master.xhtml" py:strip="True">
<head py:block="head" py:strip="True">
    <meta content="text/html; charset=UTF-8" http-equiv="Content-Type" py:if="False"/>
    <link rel="stylesheet" type="text/css" media="screen" href="${tg.url('/_pluggable/tgappcategories/css/style.css')}" />
</head>

<body py:block="body" py:strip="True">
    <div class="row">
        <div class="col-md-10 col-md-offset-1">
            <a href="${tg.url(mount_point)}" class="pull-right btn btn-primary"> Back</a>
            <div style="clear: both; margin-bottom: 10px;"></div>
        </div>
    </div>
    <div class="row">
        <div class="col-sm-10 col-sm-offset-1">
            ${form.display(values, action=action)}
        </div>
        <div py:if="values.image_small_id and h.tgappcategories.content_of_first_small_image(values)"
             class="col-sm-10 col-sm-offset-1">
            <span>Small Image</span> <span>already present</span>
        </div>
        <div py:if="values.image_big_id and h.tgappcategories.content_of_first_big_image(values)"
             class="col-sm-10 col-sm-offset-1">
            <span>Big Image</span> <span>already present</span>
        </div>
    </div>
</body>
</html>